local Lib = MissionMinderPreLoad;
local DEBUG = Lib.debug;
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

Lib.tracker = {};
local module = Lib.tracker;

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local COMPLETED = L["completed"]


local SECONDS = 1
local MINUTES = 60 * SECONDS
local HOURS = 60 * MINUTES
local DAYS = 24 * HOURS


local MAX_GARRISON_CACHE = 500
local GARRISON_CACHE_RATE_IN_SECONDS = 10 * MINUTES
local GARRISON_CACHE_WARN_AMOUNT = MAX_GARRISON_CACHE


local TIMES = { DAYS, HOURS, MINUTES}
local TIMESTRS = {INT_SPELL_DURATION_DAYS, INT_SPELL_DURATION_HOURS, INT_SPELL_DURATION_MIN }
function module.secondsToString(seconds)
	local str = ""

	if  ( seconds < MINUTES ) then
		str = str .. string.format(INT_SPELL_DURATION_SEC, seconds);
	else
		local reportOn=2
		for i=1, #TIMES do
			if ( seconds >= TIMES[i] ) then
				if(reportOn <= 0) then
					break;
				end

				if ( str:len() > 0 ) then
					str = str .. " ";
				end

				str = str .. string.format(TIMESTRS[i], seconds / TIMES[i])
				seconds = seconds % TIMES[i]
				reportOn = reportOn - 1
			end
		end
	end

	return str
end


local function getPlayerName()
	return GetUnitName("player") .. "-" .. GetRealmName()
end


function module.estimateGarrisonCache(player)
	local now = time()
	local lastLooted = Lib.Globals.Characters[player].GarrisonCache

	if ( lastLooted ) then
		return min(MAX_GARRISON_CACHE, floor((now - lastLooted) / GARRISON_CACHE_RATE_IN_SECONDS))
	else
		return nil
	end
end

local function orderBuildings(building1, building2)
	if ( building1.player ~= building2.player ) then
		return strcmputf8i(building1.player, building2.player) < 0
	elseif ( building1.isComplete ~= building2.isComplete ) then
		return building1.isComplete
	elseif ( building1.endDate ~= building2.endDate ) then
		return building1.endDate < building2.endDate
	else
		return strcmputf8i(building1.name, building2.name) < 0
	end
end

local function orderWorkOrders(workorder1, workorder2)
	local pending1 = workorder1.total - workorder1.ready
	local pending2 = workorder2.total - workorder2.ready

	if ( workorder1.player ~= workorder2.player ) then
		return strcmputf8i(workorder1.player, workorder2.player) < 0
	elseif ( pending1 ~= pending2 ) then
		return pending1 < pending2
	else
		return strcmputf8i(workorder1.name, workorder2.name) < 0
	end
end

local function orderTrackedMissions(track1, track2)
	if ( track1.completed ~= track2.completed ) then
		return track1.completed > track2.completed
	elseif ( track1.lowest ~= track2.lowest ) then
		return track1.lowest < track2.lowest
	else
		return strcmputf8i(track1.name, track2.name) < 0;
	end
end

local function showTrackMissionTooltip()
	local now = time();
	local currentPlayer = getPlayerName()

	local currentPlayerTbl = nil

	local tracking = {}
	local characters = Lib.Globals.Characters
	for player, tbl in pairs(characters) do
		local lowest = nil;
		local completed = 0

		if ( tbl.missions ) then
			for missionID, mission in pairs(tbl.missions) do
				local endDate = mission.endDate
				if ( endDate <= now ) then
					lowest = 0;
					completed = completed + 1
				elseif ( not lowest or endDate - now < lowest ) then
					lowest = endDate - now;
				end
			end
		end

		if ( lowest ) then
			local tbl = {}
			tbl.name = player
			tbl.lowest = lowest
			tbl.completed = completed

			if (player == currentPlayer ) then
				currentPlayerTbl = tbl
			else
				tinsert(tracking, tbl)
			end
		end
	end


	-- Mission progress should always show when available
	if ( currentPlayerTbl or #tracking > 0 ) then
		GameTooltip:AddLine(" ");
		GameTooltip:AddLine(GARRISON_MISSIONS);
		if ( currentPlayerTbl ) then
			if ( currentPlayerTbl.completed > 0 ) then
					GameTooltip:AddDoubleLine(currentPlayer, currentPlayerTbl.completed .. " " .. COMPLETED, 1, 1, 1, 0, 1, 0);
			else
					GameTooltip:AddDoubleLine(currentPlayer, module.secondsToString(currentPlayerTbl.lowest), 1, 1, 1, 0.7, 0.7, 0.7);
			end
		end

		table.sort(tracking, orderTrackedMissions);
		for i = 1, #tracking do
			if ( tracking[i].completed > 0 ) then
				GameTooltip:AddDoubleLine(tracking[i].name, tracking[i].completed .. " " .. COMPLETED, 1, 1, 1, 0, 1, 0);
			else
				GameTooltip:AddDoubleLine(tracking[i].name, module.secondsToString(tracking[i].lowest), 1, 1, 1, 0.7, 0.7, 0.7);
			end
		end;
	end

	-- Building progress should always show when applicable
	local buildings = {}
	for player, tbl in pairs(characters) do
		if ( tbl.buildings ) then
			for _, building in pairs(tbl.buildings) do
				if ( building.isBuilding or building.canActivate ) then
					tinsert(buildings, {
						["player"] = player,
						["name"] = building.name,
						["isComplete"] = (now >= building.endDate),
						["endDate"] = building.endDate
					})
				end
			end
		end
	end
	if ( #buildings > 0 ) then
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(GARRISON_LANDING_STATUS_BUILDING)
		table.sort(buildings, orderBuildings)

		local lastPlayer = nil
		for i = 1, #buildings do
			if ( lastPlayer ~= buildings[i].player ) then
				if ( lastPlayer ) then
					GameTooltip:AddLine(" ")
				end
				GameTooltip:AddLine(buildings[i].player)
				lastPlayer = buildings[i].player
			end

			if ( buildings[i].isComplete ) then
				GameTooltip:AddDoubleLine(buildings[i].name, COMPLETED, 1, 1, 1, 0, 1, 0);
			else
				GameTooltip:AddDoubleLine(buildings[i].name, module.secondsToString(buildings[i].endDate - now), 1, 1, 1, 0.7, 0.7, 0.7)
			end

		end
	end

	-- if any GarrisonCache is > 356 (less than 1 day from maxing)
	local garrisonCaches = {};
	for player, tbl in pairs(characters) do
		local cache = module.estimateGarrisonCache(player)
		if ( cache and cache >= GARRISON_CACHE_WARN_AMOUNT ) then
			tinsert(garrisonCaches, {
				["character"] = player,
				["amount"] = cache
			});
		end
	end
	if ( #garrisonCaches > 0 ) then
		GameTooltip:AddLine(" ");
		GameTooltip:AddLine(GARRISON_CACHE);
		table.sort(garrisonCaches, sortGarrisonCache)
		for i = 1, #garrisonCaches do
			GameTooltip:AddDoubleLine(garrisonCaches[i].player, garrisonCaches[i].amount, 1, 1, 1, 1, 0, 0);
		end
	end


	-- if any work orders are nearly complete and need to be refilled in less than a day
	local workorders = {}
	for player, tbl in pairs(characters) do
		if ( tbl.workorders ) then
			for i = 1, #tbl.workorders do
				local workorder = tbl.workorders[i]
				local name = workorder.name
				local shipmentCapacity = workorder.shipmentCapacity
				local shipmentsReady = workorder.shipmentsReady
				local shipmentsTotal = workorder.shipmentsTotal
				local creationTime = workorder.creationTime
				local duration = workorder.duration

				if ( name and shipmentCapacity > 0  and shipmentsTotal ) then
					-- Adjust ready based on elapsed time
					shipmentsReady = min(shipmentsTotal, shipmentsReady + (now - creationTime) / duration)

					local timeRemaining = (shipmentsTotal - shipmentsReady) * duration
					if ( timeRemaining < 1 * DAYS ) then
						tinsert(workorders, {
							["player"] = player,
							["name"] = name,
							["ready"] = floor(shipmentsReady),
							["total"] = shipmentsTotal
						});
					end
				end
			end
		end
	end
	if ( #workorders > 0 ) then
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(CAPACITANCE_WORK_ORDERS)
		sort(workorders, orderWorkOrders)

		local lastPlayer = nil
		for i = 1, #workorders do
			if ( lastPlayer ~= workorders[i].player ) then
				if ( lastPlayer ) then
					GameTooltip:AddLine(" ")
				end
				GameTooltip:AddLine(workorders[i].player)
				lastPlayer = workorders[i].player
			end

			GameTooltip:AddDoubleLine(workorders[i].name, workorders[i].ready .. "/" .. workorders[i].total, 1, 1, 1, 0, 1, 0);
		end
	end

	GameTooltip:Show()
end

local function trackGarrisonCache(event, quanity)
	Lib.Globals.Characters[getPlayerName()].GarrisonCache = time()
end

local function trackBuilding(event, plotID)
	local currentPlayer = getPlayerName()
	if ( not plotID ) then
		return;
	end

	local buildings = Lib.Globals.Characters[currentPlayer].buildings
	if ( not buildings ) then
		buildings = {}
		Lib.Globals.Characters[currentPlayer].buildings = buildings
	end

	local id, name, texPrefix, icon, rank, isBuilding, timeStart, buildTime, canActivate, canUpgrade, isPrebuilt = C_Garrison.GetOwnedBuildingInfoAbbrev(plotID);
	if ( isBuilding or canActivate ) then
		buildings[plotID] = {
			["plotID"] = plotID,
			["buildingID"] = id,
			["name"] = name,
			["texPrefix"] = texPrefix,
			["icon"] = icon,
			["rank"] = rank,
			["isBuilding"] = isBuilding,
			["timeStart"] = timeStart,
			["buildTime"] = buildTime,
			["endDate"] = timeStart + buildTime,
			["canActivate"] = canActivate,
			["canUpgrade"] = canUpgrade,
			["isPrebuilt"] = isPrebuilt
		}
	else
		buildings[plotID] = nil
	end
end

local function trackShipments(event)
	local currentPlayer = getPlayerName()
	local workorders = Lib.Globals.Characters[currentPlayer].workorders
	if ( not workorders ) then
		workorders = {}
		Lib.Globals.Characters[currentPlayer].workorders = workorders
	end

	Lib.Globals.Characters[currentPlayer].buildings = {}

	local workorderIndex = 1;
	local buildings = C_Garrison.GetBuildings();
	for i = 1, #buildings do
		local plotID = buildings[i].plotID
		trackBuilding(nil, plotID)

		local buildingID = buildings[i].buildingID;
		if ( buildingID) then
			local name, texture, shipmentCapacity, shipmentsReady, shipmentsTotal, creationTime, duration, timeleftString, itemName, itemIcon, itemQuality, itemID = C_Garrison.GetLandingPageShipmentInfo(buildingID);

			if ( name and shipmentCapacity > 0 ) then
				local workorder = workorders[workorderIndex]
				if ( not workorder ) then
					workorder = {}
					workorders[workorderIndex] = workorder
				end
				workorderIndex = workorderIndex + 1;

				workorder.texture = texture
				workorder.name = name
				workorder.buildingID = buildingID
				workorder.plotID = plotID;
				workorder.shipmentCapacity = shipmentCapacity
				workorder.shipmentsReady = shipmentsReady
				workorder.shipmentsTotal = shipmentsTotal
				workorder.creationTime = creationTime
				workorder.duration = duration
				workorder.itemName = itemName
			end
		end
	end
	for i = workorderIndex, #workorders do
		workorders[i] = nil
	end
end

local function initialize()
	DEBUG("   |cffffffffpreload-Tracker|r initialized");

	-- Display mission status on GarrisonReport tooltip
	GarrisonLandingPageMinimapButton:HookScript("OnEnter", showTrackMissionTooltip)

	-- Track garrison cache across multiple characters
	Lib.RegisterCallback("tracker", "GARRISON_CACHE_LOOTED", trackGarrisonCache)

	-- Shipment info
	Lib.RegisterCallback("tracker", "SHIPMENT_UPDATE_AVAILABLE", trackShipments)

	-- Building info
	Lib.RegisterCallback("tracker", "BUILDING_UPDATE_AVAILABLE", trackBuilding)
end

Lib:RegisterEvent("tracker", "ADDON_LOADED", initialize);
