local Lib = MissionMinder;
Lib.callbacks= Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local options = MissionMinder.options;
local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local DEBUG = MissionMinder.debug;

MissionMinder.blizzfix = {};
local module = MissionMinder.blizzfix;

local function hidePlacerFrame()
	if ( GarrisonFollowerPlacerFrame:IsShown() ) then
		GarrisonFollowerPlacerFrame:Hide();
	end
end

local function GenericHybridScrollMouseOverUpdate(self, ...)
	local buttons = self.buttons;
	for i = 1, #buttons do
		if ( buttons[i]:IsShown() and buttons[i]:IsMouseOver() ) then
			local func = buttons[i]:GetScript("OnEnter");
			if(func) then
				func(buttons[i]);
			end
			return;
		end
	end
end

local function hideFollowerListContextMenu(self, ...)
	GarrisonFollowerOptionDropDown.followerID = nil;
	CloseDropDownMenus();
end

local function PreloadMissionModels(...)
	local missionID = ...;
	if ( C_Garrison.GetMissionDisplayIDs(missionID) == nil ) then
		return; -- If no values returned, then return without preloading
	end

	module.MissionCompletePreload_LoadMission(...);
end

local function refreshMissionCounters(...)
	local self = GarrisonMissionFrame.MissionTab.MissionPage
	local mission = self.missionInfo;

	if ( self:IsShown() and mission ) then
		GarrisonMissionFrame.followerCounters = C_Garrison.GetBuffedFollowersForMission(mission.missionID)
		GarrisonMissionFrame.followerTraits = C_Garrison.GetFollowersTraitsForMission(mission.missionID);
		GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);
	end
end

local function initialize()
	DEBUG("   |cffffffffBlizzardFixes|r initialized");

--function module.initialize()
	-- Fix problem where StartDrag on an empty party member will cause the GarrisonFollowerPlacerFrame to show
	--and NEVER hide. This proceeds to eat all mouse events and cause the GarrisonMissionFrame to be noninteractive.
	--The placer frame uses UIParent, so is present even after closing the GarrisonMissionFrame.
	--Typically a /reload ui is required (or manually hiding the placer frame)
	if ( not MissionMinder.ptr ) then
		local partyFollowers = GarrisonMissionFrame.MissionTab.MissionPage.Followers;
		for i = 1, #partyFollowers do
			-- We reuse the GarrisonFollowerListButton code as it actually already does a proper check
			partyFollowers[i]:SetScript("OnDragStop", GarrisonFollowerListButton_OnDragStop);
		end
	end

	-- For added measure clearMouse now hides PlacerFrame if it somehow happens to be visible on accident
	hooksecurefunc("GarrisonMissionFrame_ClearMouse", hidePlacerFrame);


	-- Scrolling through various lists do not update the tooltip or context menus
	if ( not MissionMinder.ptr ) then
		GarrisonMissionFrame.MissionTab.MissionList.listScroll:HookScript("OnMouseWheel", function(self, ...)
			if(not GarrisonMissionFrame.MissionTab.MissionList.CompleteDialog:IsShown()
					and not GarrisonMissionFrame.MissionComplete:IsShown()) then
				GenericHybridScrollMouseOverUpdate(self, ...)
			end
		end);
		GarrisonLandingPageReport.List.listScroll:HookScript("OnMouseWheel", GenericHybridScrollMouseOverUpdate);
	end
	GarrisonMissionFrame.FollowerList.listScroll:HookScript("OnMouseWheel", GenericHybridScrollMouseOverUpdate);
	GarrisonMissionFrame.FollowerList.listScroll:HookScript("OnMouseWheel", hideFollowerListContextMenu);
	GarrisonLandingPage.FollowerList.listScroll:HookScript("OnMouseWheel", GenericHybridScrollMouseOverUpdate);

	-- C_Garrison.GetMissionDisplayIDs(missionID) will return nil in certain scenarios, and the blizzard code
	--does not handle nils. This case seems to occur when models have already been preloaded, and we try to
	--preload a second time.
	module.MissionCompletePreload_LoadMission = MissionCompletePreload_LoadMission;
	MissionCompletePreload_LoadMission = PreloadMissionModels;
end

local events = Lib.events;
events:RegisterEvent("blizzfixes", "ADDON_LOADED", initialize);
events:RegisterEvent("blizzfixes", "GARRISON_FOLLOWER_LIST_UPDATE", function(...)
	if ( not MissionMinder.ptr ) then
		refreshMissionCounters(...);
	end
end);
