local Lib = MissionMinder;
Lib.callbacks= Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local options = MissionMinder.options;
local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local DEBUG = MissionMinder.debug;

MissionMinder.followers = {};
local module = MissionMinder.followers;

local iWeapon = CreateFrame("Frame", nil, GarrisonLandingPage.FollowerTab, "MissionMinderItemButtonTemplate");
local iArmor = CreateFrame("Frame", nil, GarrisonLandingPage.FollowerTab, "MissionMinderItemButtonTemplate");
iWeapon:SetPoint("TOPLEFT", GarrisonLandingPage.FollowerTab.Model, "BOTTOMLEFT", 18, -3);
iArmor:SetPoint("TOPLEFT", iWeapon, "TOPRIGHT", 10, 0);


local function showItemLevel(self, followerID)
	if ( followerID and followerID ~= 0 ) then
		local follower = C_Garrison.GetFollowerInfo(followerID);
		if ( follower and follower.isCollected and follower.level == GARRISON_FOLLOWER_MAX_LEVEL ) then
			local weaponItemID, weaponItemLevel, armorItemID, armorItemLevel = C_Garrison.GetFollowerItems(followerID);
			GarrisonFollowerPage_SetItem(iWeapon, weaponItemID, weaponItemLevel);
			GarrisonFollowerPage_SetItem(iArmor, armorItemID, armorItemLevel);
			return;
		end
	end

	iWeapon:Hide();
	iArmor:Hide();
end


-- A list of abilities that have unique counters allowing us to seed the counter Icon, ID, and Name.
--Description is only available when a follower has that counter (or technically a misison has that
--mechanic, but we don't look for that)
local COUNTER_SEED = {
	114, 132, 106, 119, 170, 5, 166, 151, 158, 4, 38, 43,
	37, 36, 40, 41, 42, 39, 7, 9, 8, 45, 46, 44, 227, 49
};


local function filterFollowersForCounter(self)
	local search = GarrisonMissionFrameFollowers.SearchBox;

	if(self.info) then
		if(search:GetText() == self.info.name) then
			search:SetText("");
		else
			search:SetText(self.info.name)
		end
	end
end

local function ActiveFollowerCountersIcon_OnLeave()
	GarrisonMissionMechanicTooltip:Hide();
end


local function createActiveFollowerCountersFrame()
	local self = MissionMinderActiveFollowerCounters;

	local index = 1
	for i = 1, #COUNTER_SEED do
		local id, name, icon = C_Garrison.GetFollowerAbilityCounterMechanicInfo(COUNTER_SEED[i]);
		local frame = self.Counters[i];
		if( not frame ) then
			frame = CreateFrame("Button", nil, self, "MissionMinderActiveFollowerCounterIcon");
			frame:SetPoint("LEFT", self.Counters[i-1], "RIGHT", 5, 0);
			tinsert(self.Counters, frame);
		end

		frame.info = {
			["name"] = name,
			["icon"] = icon,
			["description"] = "" -- description cannot be seeded from ability alone
		}
		frame.mechanicID = id

		frame.Icon:SetTexture(icon);
		frame:HookScript("OnEnter", GarrisonMissionMechanic_OnEnter);
		frame:HookScript("OnLeave", ActiveFollowerCountersIcon_OnLeave);
		frame:HookScript("OnClick", filterFollowersForCounter);
		frame:Show();
	end
end


local function adjustFollowerFrame()
	local adjustment = MissionMinderActiveFollowerCounters:GetHeight();
	if (MissionMinderActiveFollowerCounters:IsShown()) then
		adjustment = - adjustment;
	end

	local self = GarrisonMissionFrame.FollowerList;
	local width, height = self:GetSize();
	point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint(1);
	self:SetSize(width, height + adjustment);
	self:ClearAllPoints();
	self:SetPoint(point, relativeTo, relativePoint, xOfs, yOfs + adjustment);

	self = GarrisonMissionFrame.FollowerTab;
	width, height = self:GetSize();
	point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint(1);
	self:SetSize(width, height + adjustment);
	self:ClearAllPoints();
	self:SetPoint(point, relativeTo, relativePoint, xOfs, yOfs + adjustment);

	self = GarrisonMissionFrame.FollowerTab.Model;
	width, height = self:GetSize();
	self:SetSize(width, height + adjustment);
end




local function updateFollowerCounters()
	if ( not MissionMinderActiveFollowerCounters:IsShown() ) then
		return
	end

	local self = MissionMinderActiveFollowerCounters
	local followers = C_Garrison.GetFollowers();

	local counters = {}
	for i = 1, #followers do
		local follower = followers[i];
		if ( follower.isCollected and (follower.status == nil or follower.status == GARRISON_FOLLOWER_ON_MISSION
				or follower.status == GARRISON_FOLLOWER_IN_PARTY)) then

			local abilities = C_Garrison.GetFollowerAbilities(follower.followerID);
			for _,ability in pairs(abilities) do
				for counterID, counter in pairs(ability.counters) do
					if ( not counters[counterID] ) then
						counters[counterID] = {};
						counters[counterID].info = counter;
					end

					counters[counterID].total = (counters[counterID].total or 0) + 1
				end
			end
		end
	end

	for index = 1, #self.Counters do
		local frame = self.Counters[index];
		local counter = counters[frame.mechanicID];
		if ( counter ) then
			frame.info = counter.info;
			frame.Text:SetText(counter.total);
			frame.Icon:SetDesaturated(false);
			frame:SetHighlightTexture("Interface\\Buttons\\CheckButtonHilight");
		else
			frame.Text:SetText("");
			frame.Icon:SetDesaturated(true);
			frame:SetHighlightTexture(nil);
		end
	end
end

local function showActiveFollowerCounterTotals()
	if ( options.ShowFollowerCounterTotals() ) then
		MissionMinderActiveFollowerCounters:Show();
	end
end

local function hideActiveFollowerCounterTotals()
	MissionMinderActiveFollowerCounters:Hide();
end

local function clearFollowerSearch()
	local search = GarrisonMissionFrameFollowers.SearchBox;
	search:SetText("");
end

local function hideBlizzardCounters()
	if ( GarrisonMissionFrame:IsShown() and options.ShowFollowerCounterTotals() ) then
		GarrisonThreatCountersFrame:Hide();
	end
end

local function initialize()
--function module.initialize()
	DEBUG("   |cffffffffFollowers|r initialized");

	iWeapon.Name:SetText(WEAPON);
	iArmor.Name:SetText(ARMOR);

	-- hide the previously existing frames (made with a different template)
	if ( GarrisonLandingPage.FollowerTab.ItemWeapon ) then
		GarrisonLandingPage.FollowerTab.ItemWeapon:Hide();
	end
	if ( GarrisonLandingPage.FollowerTab.ItemArmor ) then
		GarrisonLandingPage.FollowerTab.ItemArmor:Hide();
	end

	-- replace parentKey with our new frames
	GarrisonLandingPage.FollowerTab.ItemWeapon = iWeapon;
	GarrisonLandingPage.FollowerTab.ItemArmor = iArmor;

	hooksecurefunc("GarrisonFollowerPage_ShowFollower", showItemLevel);


	-- Adjust and update followers counters when showing the ActiveFollowerCounters frame
	--These hooks make the assumption that Show() will only call OnShow handlers if the frame was originally hidden
	--and Hide() will only call OnHide handlers if the frame was originally visible
	MissionMinderActiveFollowerCounters:HookScript("OnShow", adjustFollowerFrame);
	MissionMinderActiveFollowerCounters:HookScript("OnShow", updateFollowerCounters);
	MissionMinderActiveFollowerCounters:HookScript("OnHide", adjustFollowerFrame);

	-- Show ActiveFollowerCounter frame with FollowerTab
	GarrisonMissionFrame.FollowerTab:HookScript("OnShow", showActiveFollowerCounterTotals);
	GarrisonMissionFrame.FollowerTab:HookScript("OnHide", hideActiveFollowerCounterTotals);

	GarrisonMissionFrame:HookScript("OnHide", clearFollowerSearch);

	-- initialize frame and seed counter icons
	createActiveFollowerCountersFrame();

	if ( MissionMinder.ptr ) then
		GarrisonThreatCountersFrame:HookScript("OnShow", hideBlizzardCounters);
	end
end


local events = MissionMinder.events;
events:RegisterEvent("followers", "ADDON_LOADED", initialize)
events:RegisterEvent("followers", "GARRISON_FOLLOWER_LIST_UPDATE", updateFollowerCounters);
events:RegisterEvent("followers", "GARRISON_FOLLOWER_XP_CHANGED", updateFollowerCounters);
